import { Plugin, PluginFunction, WebSocketConfig } from '../Type';
import { Bot } from './Bot';
export declare class App {
    /**
     * BOT构造函数
     * @param name 插件名
     * @param dirname 插件设置保存位置
     */
    constructor(name?: string, dirname?: string | false, filename?: string);
    private isStart;
    private readonly Bot;
    private whitelist;
    private blacklist;
    /**
     * 增加白名单列表，请勿和黑名单同时使用
     * @param group_list 白名单列表
     */
    white(group_list: number[]): this;
    /**
     * 增加黑名单列表，请勿和白名单同时使用
     * @param group_list 黑名单列表
     */
    black(group_list: number[]): this;
    /**
     * 设置管理员
     */
    admin(id_list: number[]): this;
    noSysComm(group_list: number[]): this;
    /**
     * 增加不输出log的群组
     */
    noLog(gorup_list: number[]): this;
    private _pluginsList;
    /**
     * 载入方法插件
     * @param plugin 插件方法
     */
    plugin(plugin: PluginFunction): this;
    /**
     * 载入类插件
     * @param plugin 插件类
     * @param config 插件设置
     */
    plugin<T extends Plugin>(plugin: T, config?: InstanceType<T>['config']): this;
    /**
     * 启动函数
     * @param ws 链接设置
     * @param debug 是否开启debug
     * @param showLog 是否在控制台输出日志
     */
    start(ws?: WebSocketConfig, debug?: boolean, showLog?: boolean): Promise<Bot>;
    private getData;
    private setSysCommand;
    private initBot;
    private initPlugin;
}
