"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Admin = void 0;
const colors_1 = require("colors");
const node_schedule_1 = require("node-schedule");
/** 管理员相关 */
class Admin {
    constructor(bot) {
        this.adminList = new Set();
        /** 被ban名单 */
        this.banlist = new Set();
        this.Bot = bot;
    }
    addAdmin(admin_id) {
        this.adminList = new Set([...this.adminList, ...admin_id]);
    }
    isAdmin(id) {
        return this.adminList.has(id);
    }
    /**
     * 禁用所有功能
     * @param {number} group_id
     * @param {number} user_id
     * @param {number} time
     */
    ban(group_id, user_id, time = 1) {
        if (!user_id || this.adminList.has(user_id)) {
            return;
        }
        const msg = this.Bot.Data.config.admin.ban.replace('[time]', time.toString());
        if (group_id) {
            this.Bot.Api.sendGroupMsg(group_id, `${this.Bot.CQCode.at(user_id)}${msg}`);
        }
        else {
            this.Bot.Api.sendPrivateMsg(user_id, msg);
        }
        this.banlist.add(user_id);
        const user = group_id ? `(${group_id}) - (${user_id})` : `(${user_id})`;
        this.Bot.Log.logWarning(`${user} 被禁用 ${time} 分钟`, 'BAN');
        node_schedule_1.scheduleJob('ban' + user_id.toString(), new Date(Date.now() + time * 60 * 1000), () => {
            if (this.banlist.has(user_id)) {
                this.banlist.delete(user_id);
                this.Bot.Log.logWarning(`${user} 已解除禁用`, 'BAN');
                if (group_id) {
                    this.Bot.Api.sendGroupMsg(group_id, this.Bot.Data.config.admin.group_lift_ban.replace('[id]', this.Bot.CQCode.at(user_id)));
                }
                else {
                    this.Bot.Api.sendPrivateMsg(user_id, this.Bot.Data.config.admin.private_lift_ban);
                }
            }
        });
    }
    liftBan(user_id) {
        node_schedule_1.cancelJob('ban' + user_id.toString());
        if (this.banlist.has(user_id)) {
            this.banlist.delete(user_id);
            this.Bot.Log.logWarning(`${user_id} 已解除禁用`, 'BAN');
        }
    }
    isBan(user_id) {
        return this.banlist.has(user_id);
    }
    /** 增加白名单列表，请勿和黑名单同时使用 */
    addWhitelist(group_list) {
        if (group_list.length < 1)
            return;
        if (this.Bot.Conn.blacklist.size > 0) {
            this.Bot.Log.logWarning(`已设置黑名单，该白名单 ${colors_1.magenta(group_list.toString())} 设置无效`, 'Admin');
            return;
        }
        this.Bot.Conn.whitelist = new Set(([...this.Bot.Conn.whitelist, ...group_list]));
    }
    /** 增加黑名单列表，请勿和白名单同时使用 */
    addBlacklist(group_list) {
        if (group_list.length < 1)
            return;
        if (this.Bot.Conn.whitelist.size > 0) {
            this.Bot.Log.logWarning(`已设置白名单，该黑名单 ${colors_1.magenta(group_list.toString())} 设置无效`, 'Admin');
            return;
        }
        this.Bot.Conn.blacklist = new Set(([...this.Bot.Conn.blacklist, ...group_list]));
    }
}
exports.Admin = Admin;
