"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setCommLister = exports.SetComm = exports.Comm = exports.Command = void 0;
const colors_1 = require("colors");
class Command {
    constructor(Bot) {
        /** 命令列表 */
        this.list = [];
        this.Bot = Bot;
    }
    /** 增加命令 */
    command(name) {
        const repeat = this.list.some(item => {
            return item.comm.includes(name);
        });
        if (repeat) {
            this.Bot.Log.logWarning(`发现重复指令 ${colors_1.white(name)}, 指令设置失败`, '指令');
        }
        const comm = new Comm(name);
        this.list.push(comm);
        exports.setCommLister(this.Bot, comm);
        this.Bot.Log.logNotice(`${comm.group ? `${colors_1.yellow(comm.group)} - ` : ''}${colors_1.yellow(name)} 已加载`, '指令');
        return new SetComm(this.Bot, comm, repeat);
    }
}
exports.Command = Command;
class Comm {
    constructor(name) {
        this.comm = [];
        this.admin = false;
        this.desc = '暂无描述';
        this.fn = {
            group: [],
            private: []
        };
        this.whitelist = new Set();
        this.blacklist = new Set();
        this.comm.push(name);
    }
}
exports.Comm = Comm;
class SetComm {
    constructor(Bot, comm, repeat, plugin = false) {
        this.Bot = Bot;
        this.comm = comm;
        this.repeat = repeat;
        this.plugin = plugin;
    }
    /** 增加指令别名 */
    alias(name) {
        const repeat = this.Bot.Command.list.some(item => {
            return item.comm.includes(name);
        });
        if (repeat) {
            this.Bot.Log.logWarning(`别名与其它指令重复 ${colors_1.white(name)}, 别名设置失败`, '指令');
        }
        else {
            this.comm.comm.push(name);
        }
    }
    /** 增加命令描述，重复调用会被覆盖 */
    desc(text) {
        this.comm.desc = text;
        return this;
    }
    /** 增加正则规则，调用后命令名将会失效 */
    reg(reg) {
        if (this.repeat)
            return this;
        this.comm.reg = reg;
        this.comm.desc += '(动态指令)';
        return this;
    }
    /** 是否为管理员指令 */
    admin(flag = true) {
        this.comm.admin = flag;
        return this;
    }
    /** 设置命令分类 */
    group(name) {
        this.comm.group = name;
        return this;
    }
    action(type, fn) {
        if (this.repeat)
            return this;
        if (type === 'group') {
            this.comm.fn.group.push(fn);
        }
        else if (type === 'private') {
            this.comm.fn.private.push(fn);
        }
        return this;
    }
    /** 增加白名单列表，请勿和黑名单同时使用 */
    white(list) {
        if (this.repeat || list.length < 1)
            return this;
        if (this.comm.blacklist.size > 0) {
            this.Bot.Log.logWarning(`${colors_1.magenta(this.comm.comm[0])} 已设置黑名单，该白名单 ${colors_1.magenta(list.toString())} 设置无效`, this.plugin ? '插件' : 'Admin');
            return this;
        }
        this.comm.whitelist = new Set(([...this.comm.whitelist, ...list]));
        return this;
    }
    /** 增加黑名单列表，请勿和白名单同时使用 */
    black(list) {
        if (this.repeat || list.length < 1)
            return this;
        if (this.comm.whitelist.size > 0) {
            this.Bot.Log.logWarning(`${colors_1.magenta(this.comm.comm[0])} 已设置白名单，该黑名单 ${colors_1.magenta(list.toString())} 设置无效`, this.plugin ? '插件' : 'Admin');
            return this;
        }
        this.comm.blacklist = new Set(([...this.comm.blacklist, ...list]));
        return this;
    }
}
exports.SetComm = SetComm;
exports.setCommLister = (Bot, comm) => {
    Bot.Event.on('message.group', async (e) => {
        if (comm.fn.group.length < 1)
            return;
        if (comm.admin && !Bot.Admin.isAdmin(e.sender.user_id))
            return;
        if ((comm.blacklist.size > 0 && comm.blacklist.has(e.group_id)) || (comm.whitelist.size > 0 && !comm.whitelist.has(e.group_id)))
            return;
        if ((comm.reg && !comm.reg.test(e.message)) || (!comm.reg && !comm.comm.includes(e.message)))
            return;
        Bot.Log.logNotice(`群${colors_1.white(Bot.Data.groupList[e.group_id] || '')}(${colors_1.white(e.group_id.toString())}) - ${colors_1.white(e.sender.card || e.sender.nickname)}(${colors_1.white(e.user_id.toString())})触发${colors_1.yellow(comm.comm[0])}指令`, '指令');
        for (const i in comm.fn.group) {
            if (await comm.fn.group[i](e))
                return true;
        }
    });
    Bot.Event.on('message.private', async (e) => {
        if (comm.fn.private.length < 1)
            return;
        if (comm.admin && !Bot.Admin.isAdmin(e.sender.user_id))
            return;
        if ((comm.blacklist.size > 0 && comm.blacklist.has(e.sender.user_id)) || (comm.whitelist.size > 0 && !comm.whitelist.has(e.sender.user_id)))
            return;
        if ((comm.reg && !comm.reg.test(e.message)) || (!comm.reg && !comm.comm.includes(e.message)))
            return;
        Bot.Log.logNotice(`${colors_1.white(e.sender.nickname)}(${colors_1.white(e.user_id.toString())})触发${colors_1.yellow(comm.comm[0])}指令`, '指令');
        for (const i in comm.fn.private) {
            if (await comm.fn.private[i](e))
                return true;
        }
    });
};
