"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Data = void 0;
class Data {
    constructor(Bot, name) {
        /**
         * Bot的登录QQ
         */
        this.userId = -1;
        /**
         * Bot的配置
         */
        this.config = {
            admin: {
                ban: '无路赛，禁用你所有功能[time]分钟',
                group_lift_ban: '[id]放过你了，下次别这样了',
                private_lift_ban: '放过你了，下次别这样了'
            }
        };
        /**
         * 群名缓存
         */
        this.groupList = {};
        /**
         * 群员昵称缓存
         */
        this.groupMemberList = {};
        /**
         * 好友昵称缓存
         */
        this.friendList = {};
        /**
         * 不显示Log输出的群组列表
         */
        this.noLogList = new Set();
        /**
         * 不启用内置指令的群组列表
         */
        this._noCommList = new Set();
        this.Bot = Bot;
        this.name = name;
    }
    /**
     * 更新群名缓存
     */
    async updateGroupsList() {
        const list = await this.Bot.Api.getGroupList();
        list.forEach(group => {
            this.groupList[group.group_id] = group.group_name;
        });
        this.Bot.Log.logNotice('群名缓存已更新', '缓存');
    }
    /**
     * 更新指定群组群员昵称缓存
     */
    async updateGroupMemberList(group_id) {
        const list = await this.Bot.Api.getGroupMemberList(group_id);
        this.groupMemberList[group_id] = {};
        list.forEach(user => {
            this.groupMemberList[group_id][user.user_id] = user.card || user.nickname;
        });
    }
    /**
     * 更新所有群组群员昵称缓存
     */
    async updateAllGroupMemberList() {
        for (const i in this.groupList) {
            await this.updateGroupMemberList(Number(i));
        }
        this.Bot.Log.logNotice('群员昵称缓存已更新', '缓存');
    }
    async updateFriendList() {
        const list = await this.Bot.Api.getFriendList();
        list.forEach(user => {
            this.friendList[user.user_id] = user.remark || user.nickname;
        });
        this.Bot.Log.logNotice('好友昵称缓存已更新', '缓存');
    }
    /**
     * 增加不显示Log输出的群组
     */
    setNoLog(list) {
        this.noLogList = new Set(([...this.noLogList, ...list]));
    }
    set noCommList(list) {
        this._noCommList = list;
        this.Bot.Command.list.forEach(item => {
            if (item.group === '内置指令') {
                item.blacklist = this._noCommList;
            }
        });
    }
    get noCommList() {
        return this._noCommList;
    }
    /**
     * 增加不启用内置指令的群组
     */
    setNoComm(list) {
        this.noCommList = new Set(([...this._noCommList, ...list]));
    }
}
exports.Data = Data;
