"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Event = void 0;
const colors_1 = require("colors");
const Emitter_1 = require("./Emitter");
class Event {
    constructor(Bot) {
        this.init = () => {
            if (this.Bot.Data.showLog) {
                this
                    .on('message.private', event => {
                    if (event.user_id !== this.Bot.Data.userId) {
                        const name = colors_1.white(event.sender.nickname);
                        const user_id = colors_1.white(event.user_id.toString());
                        const msg = colors_1.white(event.message);
                        const message_id = colors_1.white(event.message_id.toString());
                        this.Bot.Log.logInfoRecv(`收到${name}(${user_id})的消息: ${msg} (${message_id})`, 'EVENT');
                    }
                    return this.Bot.Admin.isBan(event.user_id);
                }, true)
                    .on('message.group', event => {
                    if (!this.Bot.Data.noLogList.has(event.group_id)) {
                        const group_name = colors_1.cyan(this.Bot.Data.groupList[event.group_id] || '');
                        const group_id = colors_1.cyan(event.group_id.toString());
                        const user_name = colors_1.cyan(event.sender.card || event.sender.nickname);
                        const user_id = colors_1.cyan(event.user_id.toString());
                        const msg = colors_1.cyan(event.message);
                        const message_id = colors_1.cyan(event.message_id.toString());
                        this.Bot.Log.logInfoRecv(`收到群${group_name}(${group_id}) - ${user_name}(${user_id})的消息: ${msg} (${message_id})`, 'EVENT');
                    }
                    return this.Bot.Admin.isBan(event.user_id);
                }, true)
                    .on('notice.group_upload', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = colors_1.white(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.white(event.group_id.toString());
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.white(this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '') : '';
                    const user_id = colors_1.white(event.user_id.toString());
                    const filename = colors_1.white(event.file.name);
                    this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${user_name}(${user_id}) 上传了 ${filename}`, 'EVENT');
                })
                    .on('notice.group_admin', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = this.Bot.Data.groupList[event.group_id] || '';
                    const group_id = event.group_id.toString();
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '' : '';
                    const user_id = event.user_id.toString();
                    if (event.sub_type === 'set') {
                        this.Bot.Log.logNotice(`群${colors_1.white(group_name)}(${colors_1.white(group_id)}) - ${colors_1.white(user_name)}(${colors_1.white(user_id)}) 成为了管理员`, 'EVENT');
                    }
                    else if (event.sub_type === 'unset') {
                        this.Bot.Log.logWarning(`群${colors_1.magenta(group_name)}(${colors_1.magenta(group_id)}) - ${colors_1.white(user_name)}(${colors_1.magenta(user_id)}) 被取消了管理员`, 'EVENT');
                    }
                })
                    .on('notice.group_decrease', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = colors_1.magenta(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.magenta(event.group_id.toString());
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.magenta(this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '') : '';
                    const user_id = colors_1.magenta(event.user_id.toString());
                    const operator_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.magenta(this.Bot.Data.groupMemberList[event.group_id][event.operator_id] || '') : '';
                    const operator_id = colors_1.magenta(event.operator_id.toString());
                    if (event.sub_type === 'kick') {
                        this.Bot.Log.logWarning(`群${group_name}(${group_id}) - ${user_name}(${user_id}) 被 ${operator_name}(${operator_id}) 踢出群聊`, 'EVENT');
                    }
                    else if (event.sub_type === 'leave') {
                        this.Bot.Log.logWarning(`群${group_name}(${group_id}) - ${user_name}(${user_id}) 退出了群聊`, 'EVENT');
                    }
                    else if (event.sub_type === 'kick_me') {
                        this.Bot.Log.logWarning(`群${group_name}(${group_id}) - ${colors_1.red(this.Bot.Data.name)} 被 ${operator_name}(${operator_id}) 踢出群聊`, 'EVENT');
                    }
                })
                    .on('notice.group_increase', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = colors_1.white(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.white(event.group_id.toString());
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.white(this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '') : '';
                    const user_id = colors_1.white(event.user_id.toString());
                    const operator_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.white(this.Bot.Data.groupMemberList[event.group_id][event.operator_id] || '') : '';
                    const operator_id = colors_1.white(event.operator_id.toString());
                    const type = event.sub_type === 'invite' ? '邀请' : '同意';
                    this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${operator_name}(${operator_id}) ${type} ${user_name}(${user_id}) 加入群聊`, 'EVENT');
                })
                    .on('notice.group_ban', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = this.Bot.Data.groupList[event.group_id] || '';
                    const group_id = event.group_id.toString();
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '' : '';
                    const user_id = event.user_id.toString();
                    const operator_name = this.Bot.Data.groupMemberList[event.group_id] ? this.Bot.Data.groupMemberList[event.group_id][event.operator_id] || '' : '';
                    const operator_id = event.operator_id.toString();
                    if (event.sub_type === 'ban') {
                        this.Bot.Log.logWarning(`群${colors_1.magenta(group_name)}(${colors_1.magenta(group_id)}) - ${colors_1.magenta(user_name)}(${colors_1.magenta(user_id)}) 被 ${colors_1.magenta(operator_name)}(${colors_1.magenta(operator_id)}) 禁言 ${colors_1.magenta(event.duration.toString())}秒`, 'EVENT');
                    }
                    else if (event.sub_type === 'lift_ban') {
                        this.Bot.Log.logNotice(`群${colors_1.white(group_name)}(${colors_1.white(group_id)}) - ${colors_1.white(user_name)}(${colors_1.white(user_id)}) 被 ${colors_1.white(operator_name)}(${colors_1.white(operator_id)}) 解除禁言`, 'EVENT');
                    }
                })
                    .on('notice.friend_add', event => {
                    this.Bot.Log.logNotice(`与 (${colors_1.white(event.user_id.toString())}) 成为好友`, 'EVENT');
                })
                    .on('notice.group_recall', (event) => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = colors_1.magenta(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.magenta(event.group_id.toString());
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.magenta(this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '') : '';
                    const user_id = colors_1.magenta(event.user_id.toString());
                    const operator_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.magenta(this.Bot.Data.groupMemberList[event.group_id][event.operator_id] || '') : '';
                    const operator_id = colors_1.magenta(event.operator_id.toString());
                    const message_id = colors_1.magenta(event.message_id.toString());
                    this.Bot.Log.logWarning(`群${group_name}(${group_id}) - ${operator_name}(${operator_id}) 撤回了${event.operator_id === event.user_id ? '' : ` ${user_name}(${user_id}) 的`}一条消息 (${message_id})`, 'EVENT');
                })
                    .on('notice.friend_recall', event => {
                    const user_name = colors_1.magenta(this.Bot.Data.friendList[event.user_id] || '');
                    const user_id = colors_1.magenta(event.user_id.toString());
                    const message_id = colors_1.magenta(event.message_id.toString());
                    this.Bot.Log.logWarning(`${user_name}(${user_id}) 撤回了一条消息 (${message_id})`, 'EVENT');
                })
                    .on('notice.notify', event => {
                    if ((this.Bot.Data.noLogList.has(event.group_id)) || !event.group_id)
                        return;
                    const group_name = colors_1.white(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.white(event.group_id.toString());
                    const user_name = this.Bot.Data.groupMemberList[event.group_id] ? colors_1.white(this.Bot.Data.groupMemberList[event.group_id][event.user_id] || '') : '';
                    const user_id = colors_1.white(event.user_id.toString());
                    const target_name = event.sub_type !== 'honor' && this.Bot.Data.groupMemberList[event.group_id] ? colors_1.white(this.Bot.Data.groupMemberList[event.group_id][event.target_id] || '') : '';
                    const target_id = event.sub_type !== 'honor' ? colors_1.white(event.target_id.toString()) : '';
                    if (event.sub_type === 'poke') {
                        this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${user_name}(${user_id}) 戳了戳 ${target_name}(${target_id})`, 'EVENT');
                    }
                    else if (event.sub_type === 'lucky_king') {
                        this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${target_name}(${target_id}) 获得了 ${user_name}(${user_id}) 发出的群红包运气王`, 'EVENT');
                    }
                    else if (event.sub_type === 'honor') {
                        const honor = event.honor_type === 'talkative' ? '龙王' : event.honor_type === 'performer' ? '群聊之火' : event.honor_type === 'emotion' ? '快乐源泉' : '群荣誉';
                        this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${user_name}(${user_id}) 获得了 ${honor}`, 'EVENT');
                    }
                })
                    .on('notice.group_card', event => {
                    if (this.Bot.Data.noLogList.has(event.group_id))
                        return;
                    const group_name = colors_1.white(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.white(event.group_id.toString());
                    const user_id = colors_1.white(event.user_id.toString());
                    const oldcard = colors_1.white(event.card_old);
                    const newcard = colors_1.white(event.card_new);
                    this.Bot.Log.logNotice(`群${group_name}(${group_id}) - ${oldcard}(${user_id}) 将群名片修改为${newcard}`, 'EVENT');
                })
                    .on('notice.offline_file', event => {
                    const user_name = colors_1.white(this.Bot.Data.friendList[event.user_id] || '');
                    const user_id = colors_1.white(event.user_id.toString());
                    const filename = colors_1.white(event.file.name);
                    const url = colors_1.white(event.file.url);
                    this.Bot.Log.logNotice(`${user_name}(${user_id}) 发送了离线文件 ${filename}(${url})`, 'EVENT');
                })
                    .on('request.friend', async (event) => {
                    const user_id = colors_1.white(event.user_id.toString());
                    const comment = colors_1.white(event.comment);
                    this.Bot.Log.logNotice(`(${user_id}) 申请添加好友 验证消息：${comment}`, 'EVENT');
                })
                    .on('request.group', async (event) => {
                    const user_id = colors_1.white(event.user_id.toString());
                    const group_name = colors_1.white(this.Bot.Data.groupList[event.group_id] || '');
                    const group_id = colors_1.white(event.group_id.toString());
                    const comment = colors_1.white(event.comment);
                    if (event.sub_type === 'add') {
                        this.Bot.Log.logNotice(`(${user_id}) 申请加入 ${group_name}(${group_id}) 验证消息：${comment}`, 'EVENT');
                    }
                    else if (event.sub_type === 'invite') {
                        this.Bot.Log.logNotice(`(${user_id}) 邀请${this.Bot.Data.name}加入 ${group_name}(${group_id}) 验证消息：${comment}`, 'EVENT');
                    }
                })
                    .on('notice.client_status', event => {
                    if (event.online) {
                        this.Bot.Log.logNotice(`账号在 ${colors_1.white(event.client.device_name)} 上线`, 'EVENT');
                    }
                    else {
                        this.Bot.Log.logWarning(`账号在 ${colors_1.magenta(event.client.device_name)} 下线`, 'EVENT');
                    }
                })
                    .on('notice.essence', event => {
                    if (event.sub_type === 'add') {
                        this.Bot.Log.logNotice(`(${colors_1.white(event.sender_id.toString())}) 的消息被 (${colors_1.white(event.operator_id.toString())}) 设为精华消息 (${colors_1.white(event.message_id.toString())})`, 'EVENT');
                    }
                    else {
                        this.Bot.Log.logWarning(`(${colors_1.magenta(event.sender_id.toString())}) 的消息被 (${colors_1.magenta(event.operator_id.toString())}) 取消精华消息 (${colors_1.magenta(event.message_id.toString())})`, 'EVENT');
                    }
                });
                this.Bot.Log.logInfo('开始监听事件', 'EVENT');
            }
        };
        this.Bot = Bot;
        this.init();
    }
    on(type, fn, log) {
        this.Bot.Conn.addEvent(type, fn, log);
        return this;
    }
    onSendMessage(type, fn) {
        Emitter_1.emitter.on(type, (arg1, arg2) => {
            fn(arg1, arg2);
        });
        return this;
    }
}
exports.Event = Event;
