"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plugin = void 0;
const fs_extra_1 = require("fs-extra");
const node_yaml_1 = require("node-yaml");
const path = require("path");
const lodash_1 = require("lodash");
class Plugin {
    constructor(bot, dir, filename) {
        /** 插件列表 */
        this.list = [];
        this.saveFlag = false;
        this.Bot = bot;
        this.dirname = dir;
        this.filename = filename;
    }
    setConfig(name, config) {
        const plugin = this.getPlugin(name);
        if (plugin && plugin.name === name) {
            plugin['config'] = lodash_1.merge(plugin['config'], config);
        }
    }
    getConfig(name, key) {
        const plugin = this.getPlugin(name);
        if (key) {
            return plugin && plugin['config'] && plugin['config'][key] ? plugin['config'][key] : null;
        }
        else {
            return plugin && plugin['config'] ? plugin['config'] : null;
        }
    }
    saveConfig() {
        if (this.saveFlag) {
            this.Bot.Log.logWarning('插件配置保存任务冲突', '插件');
            return;
        }
        this.saveFlag = true;
        if (this.dirname) {
            try {
                if (!fs_extra_1.existsSync(this.dirname)) {
                    fs_extra_1.mkdirSync(this.dirname);
                }
                let config = { plugin: {} };
                this.list.forEach(plugin => {
                    if (plugin['config'] && Object.keys(plugin['config']).length > 0) {
                        config.plugin[plugin.name] = plugin['config'];
                    }
                });
                const ymlPath = path.join(this.dirname, `./${this.filename}.yml`);
                if (fs_extra_1.existsSync(ymlPath)) {
                    config = lodash_1.merge(node_yaml_1.readSync(ymlPath), config);
                }
                node_yaml_1.writeSync(path.join(this.dirname, `./${this.filename}.yml`), config);
            }
            catch (err) {
                console.error(err);
                this.Bot.Log.logError('插件配置未保存', '插件');
            }
        }
        this.saveFlag = false;
    }
    getPlugin(name) {
        const plugin = this.list.find(i => i.name === name);
        if (plugin) {
            return plugin;
        }
        else {
            this.Bot.Log.logDebug(`未找到 ${name}`, '插件');
            return undefined;
        }
    }
}
exports.Plugin = Plugin;
