"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Command = void 0;
const colors_1 = require("colors");
const Command_1 = require("../Bot/modules/Command");
class Command {
    constructor(group, Bot) {
        this._whitelist = new Set();
        this._blacklist = new Set();
        /** 增加命令 */
        this.command = (name) => {
            const repeat = this.Bot.Command.list.some(item => {
                return item.comm.includes(name);
            });
            if (repeat) {
                this.Bot.Log.logWarning(`发现重复指令 ${colors_1.white(name)}, 指令设置失败`, '指令');
            }
            const comm = new Command_1.Comm(name);
            comm.group = this.group;
            comm.whitelist = this._whitelist;
            comm.blacklist = this._blacklist;
            this.Bot.Command.list.push(comm);
            Command_1.setCommLister(this.Bot, comm);
            this.Bot.Log.logNotice(`${colors_1.yellow(this.group)} - ${colors_1.yellow(name)} 已加载`, '指令');
            return new Command_1.SetComm(this.Bot, comm, repeat);
        };
        this.group = group;
        this.Bot = Bot;
    }
    set whitelist(list) {
        this._whitelist = new Set(([...this._whitelist, ...list]));
        this.Bot.Command.list.forEach(item => {
            if (item.group === this.group) {
                item.whitelist = new Set([...this._whitelist, ...item.whitelist]);
            }
        });
    }
    /** 为插件指令增加白名单列表，请勿和黑名单同时使用 */
    white(list) {
        if (list.length < 1)
            return this;
        if (this._blacklist.size > 0) {
            this.Bot.Log.logWarning(`${colors_1.magenta(this.group)} 已设置黑名单，该白名单${colors_1.magenta(list.toString())}设置无效`, 'Admin');
            return this;
        }
        this.whitelist = list;
        return this;
    }
    /** 获取白名单 */
    getWhitelist() {
        return [...this._whitelist];
    }
    /** 移除白名单 */
    removeWhitelist(list) {
        for (const id of list) {
            this.Bot.Command.list.forEach(item => {
                if (item.group === this.group) {
                    item.whitelist.delete(id);
                }
            });
        }
    }
    set blacklist(list) {
        this._blacklist = new Set(([...this._blacklist, ...list]));
        this.Bot.Command.list.forEach(item => {
            if (item.group === this.group) {
                item.blacklist = new Set([...this._blacklist, ...item.blacklist]);
            }
        });
    }
    /** 为插件指令增加黑名单列表，请勿和白名单同时使用 */
    black(list) {
        if (list.length < 1)
            return this;
        if (this._whitelist.size > 0) {
            this.Bot.Log.logWarning(`${colors_1.magenta(this.group)} 已设置白名单，该黑名单 ${colors_1.magenta(list.toString())} 设置无效`, 'Admin');
            return this;
        }
        this.blacklist = list;
        return this;
    }
    /** 获取黑名单 */
    getBlacklist() {
        return [...this._blacklist];
    }
    /** 移除黑名单 */
    removeBlacklist(list) {
        for (const id of list) {
            this.Bot.Command.list.forEach(item => {
                if (item.group === this.group) {
                    item.blacklist.delete(id);
                }
            });
        }
    }
}
exports.Command = Command;
