import { Bot } from '../Bot/Bot';
import { Command } from './Command';
import { Log } from '../Tools';
export declare abstract class BotPlugin {
    constructor(bot: Bot);
    /** 插件名称 */
    readonly name: string;
    private _config;
    private _configProxy;
    /** 插件设置 */
    config: {
        [key: string]: any;
    };
    /** Bot对象 */
    protected Bot: Bot;
    /**
     * 插件命令
     * 与Bot.Command基本相同，但能设置统一设置
     */
    protected Command: Command;
    /**
     * 日志对象
     */
    protected Log: Log;
    /**
     * 安装方法
     * 该方法会在实例化后执行，请勿重复执行
     */
    setup(config?: any): void;
    /**
     * 初始化方法
     * 该方法会在Bot初始化完成后执行，请勿重复执行
     */
    init(): void | Promise<void>;
    private deepProxy;
    /**
     * 创建设置代理，在设置修改后自动保存本地配置
     */
    autoSave(): void;
}
