"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BotPlugin = void 0;
const Command_1 = require("./Command");
const lodash_1 = require("lodash");
const Tools_1 = require("../Tools");
class BotPlugin {
    constructor(bot) {
        /** 插件名称 */
        this.name = '未命名插件';
        /** 插件设置 */
        this.config = {};
        this.Bot = bot;
    }
    /**
     * 安装方法
     * 该方法会在实例化后执行，请勿重复执行
     */
    setup(config) {
        if (config) {
            this.config = lodash_1.merge(this.config, config);
        }
        this.Command = new Command_1.Command(this.name, this.Bot);
        this.Log = new Tools_1.Log(this.name);
    }
    /**
     * 初始化方法
     * 该方法会在Bot初始化完成后执行，请勿重复执行
     */
    init() { }
    deepProxy(obj) {
        if (typeof obj === 'object') {
            for (let key in obj) {
                if (typeof obj[key] === 'object') {
                    obj[key] = this.deepProxy(obj[key]);
                }
            }
        }
        return new Proxy(obj, {
            set: (target, key, value, receiver) => {
                if (typeof value === 'object') {
                    value = this.deepProxy(value);
                }
                const flag = Reflect.set(target, key, value, receiver);
                if (flag) {
                    this.Bot.Plugin.saveConfig();
                }
                return flag;
            },
            deleteProperty(target, key) {
                return Reflect.deleteProperty(target, key);
            }
        });
    }
    /**
     * 创建设置代理，在设置修改后自动保存本地配置
     */
    autoSave() {
        this._config = this.config;
        this._configProxy = this.deepProxy(this._config);
        Object.defineProperty(this, 'config', {
            get: () => {
                return this._configProxy;
            },
            set: (value) => {
                this._config = value;
            }
        });
    }
}
exports.BotPlugin = BotPlugin;
