/**
 * 日志级别常量
 */
export declare enum LogColor {
    /**
     * 级别：调试
     * 颜色：灰色
     */
    DEBUG = 0,
    /**
     * 级别：信息
     * 颜色：白色
     */
    INFO = 1,
    /**
     * 级别：通知
     * 颜色：青色
     */
    INFO_NOTICE = 2,
    /**
     * 级别：信息(接收)
     * 颜色：蓝色
     */
    INFO_RECV = 3,
    /**
     * 级别：信息(发送)
     * 颜色：绿色
     */
    INFO_SEND = 4,
    /**
     * 级别：警告
     * 颜色：黄色
     */
    WARNING = 5,
    /**
     * 级别：错误
     * 颜色：红色
     */
    ERROR = 6,
    /**
     * 级别：致命错误
     * 颜色：品红
     */
    FATAL = 7
}
export declare class Log {
    private name;
    constructor(name?: string);
    private printLog;
    /**
     * 在控制台调试输出日志。不推荐使用本方法,请使用log开头的方法
     * @param {LogLevel} level
     * @param {string} type
     * @param {string} content
     */
    sendLog(level: LogColor, content: string, type?: string): void;
    /**
     * 调试日志
     * @param {string} type
     * @param {string} content
     */
    logDebug(content: string, type?: string): void;
    /**
     * 信息日志
     * @param {string} type
     * @param {string} content
     */
    logInfo(content: string, type?: string): void;
    /**
     * 接受信息日志
     * @param {string} type
     * @param {string} content
     */
    logInfoRecv(content: string, type?: string): void;
    /**
     * 发送日志
     * @param {string} type
     * @param {string} content
     */
    logInfoSend(content: string, type?: string): void;
    /**
     * 通知日志
     * @param {string} type
     * @param {string} content
     */
    logNotice(content: string, type?: string): void;
    /**
     * 警告日志
     * @param {string} type
     * @param {string} content
     */
    logWarning(content: string, type?: string): void;
    /**
     * 错误日志
     * @param {string} type
     * @param {string} content
     */
    logError(content: string, type?: string): void;
    /**
     * 致命错误日志
     * @param {string} type
     * @param {string} content
     */
    logFatal(content: string, type?: string): void;
}
export declare const PrintLog: Log;
