"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrintLog = exports.Log = exports.LogColor = void 0;
const colors_1 = require("colors");
const Tools_1 = require("./Tools");
const moment = require("moment");
/**
 * 日志级别常量
 */
var LogColor;
(function (LogColor) {
    /**
     * 级别：调试
     * 颜色：灰色
     */
    LogColor[LogColor["DEBUG"] = 0] = "DEBUG";
    /**
     * 级别：信息
     * 颜色：白色
     */
    LogColor[LogColor["INFO"] = 1] = "INFO";
    /**
     * 级别：通知
     * 颜色：青色
     */
    LogColor[LogColor["INFO_NOTICE"] = 2] = "INFO_NOTICE";
    /**
     * 级别：信息(接收)
     * 颜色：蓝色
     */
    LogColor[LogColor["INFO_RECV"] = 3] = "INFO_RECV";
    /**
     * 级别：信息(发送)
     * 颜色：绿色
     */
    LogColor[LogColor["INFO_SEND"] = 4] = "INFO_SEND";
    /**
     * 级别：警告
     * 颜色：黄色
     */
    LogColor[LogColor["WARNING"] = 5] = "WARNING";
    /**
     * 级别：错误
     * 颜色：红色
     */
    LogColor[LogColor["ERROR"] = 6] = "ERROR";
    /**
     * 级别：致命错误
     * 颜色：品红
     */
    LogColor[LogColor["FATAL"] = 7] = "FATAL";
    /**
     * 级别：致命错误
     * 颜色：品红
     */
})(LogColor = exports.LogColor || (exports.LogColor = {}));
class Log {
    constructor(name = '') {
        this.name = name;
    }
    printLog(msg, level = 0) {
        if (msg.length > 500) {
            msg = msg.slice(0, 500) + '...';
        }
        const time = moment(new Date()).format('YYYY-MM-DD HH:mm:ss');
        switch (level) {
            case LogColor.DEBUG:
                console.log(time, '->', colors_1.gray(msg));
                break;
            case LogColor.INFO:
                console.log(time, '->', msg);
                break;
            case LogColor.INFO_NOTICE:
                console.log(time, '->', colors_1.cyan(msg));
                break;
            case LogColor.INFO_RECV:
                console.log(time, '->', colors_1.blue(msg));
                break;
            case LogColor.INFO_SEND:
                console.log(time, '->', colors_1.green(msg));
                break;
            case LogColor.WARNING:
                console.warn(time, '->', colors_1.yellow(msg));
                break;
            case LogColor.ERROR:
                console.error(time, '->', colors_1.red(msg));
                break;
            case LogColor.FATAL:
                console.error(time, '->', colors_1.magenta(msg));
                break;
        }
    }
    /**
     * 在控制台调试输出日志。不推荐使用本方法,请使用log开头的方法
     * @param {LogLevel} level
     * @param {string} type
     * @param {string} content
     */
    sendLog(level, content, type) {
        this.printLog(`[${type || this.name || '日志'}] ${Tools_1.decode(content)}`, level);
    }
    /**
     * 调试日志
     * @param {string} type
     * @param {string} content
     */
    logDebug(content, type = 'DEBUG') {
        this.sendLog(LogColor.DEBUG, content, type);
    }
    /**
     * 信息日志
     * @param {string} type
     * @param {string} content
     */
    logInfo(content, type) {
        this.sendLog(LogColor.INFO, content, type);
    }
    /**
     * 接受信息日志
     * @param {string} type
     * @param {string} content
     */
    logInfoRecv(content, type) {
        this.sendLog(LogColor.INFO_RECV, content, type);
    }
    /**
     * 发送日志
     * @param {string} type
     * @param {string} content
     */
    logInfoSend(content, type) {
        this.sendLog(LogColor.INFO_SEND, content, type);
    }
    /**
     * 通知日志
     * @param {string} type
     * @param {string} content
     */
    logNotice(content, type) {
        this.sendLog(LogColor.INFO_NOTICE, content, type);
    }
    /**
     * 警告日志
     * @param {string} type
     * @param {string} content
     */
    logWarning(content, type) {
        this.sendLog(LogColor.WARNING, content, type);
    }
    /**
     * 错误日志
     * @param {string} type
     * @param {string} content
     */
    logError(content, type) {
        this.sendLog(LogColor.ERROR, content, type);
    }
    /**
     * 致命错误日志
     * @param {string} type
     * @param {string} content
     */
    logFatal(content, type) {
        this.sendLog(LogColor.FATAL, content, type);
    }
}
exports.Log = Log;
exports.PrintLog = new Log();
